# cdd.py - Data handling routines developed by Optibrium for integration  of CDD Vault with StarDrop software
# Use of this code is subject to the terms of the StarDrop License Agreement and use, copying or redistribution of this file for use by other persons or organisations is prohibited
# Copyright(C) Optibrium Ltd 2016
# stardrop-support@optibrium.com
#

"""Module to control access to the CDD Vault from StarDrop"""

import urllib2
import json
import time
import sys
import app
import PythonQt
import cddconfig
import CSVHelper


try:
	import refreshscript

	def refreshCDDSearch(method,params):
		"""Run a search and supply the information to StarDrop to reform a refresh operation.
		@param: method: Should be 'CDD'.
		@param: params: The parameters to sue when running the search.
		"""
		print 'refreshing CDD'
		if method == "CDD":
			ds = runSearch(None,int(params.get("search","")),int(params.get("vault","")),params.get("projects",""),params.get("token",""))
		return ds.pack()

	refreshscript.setRefreshFunction(refreshCDDSearch)
except:
	pass
	
token = cddconfig.getToken()
invalidvaultid = -1 # Invalid vault ID to be used as a default
invalidprojectids = []  # Empty list of projects to be used as a default
invalidsearchid = -1 # Invalid search ID to be used as a default
usercancelled = -2
	
def callServer(url,token,post = False):
	"""Contact the CDD Vault and run the specified operation.
	@param: url: The url of the resource being accessed.
	@param: token: The API token to be used, as a string.
	@param: post: A flag indicating whether this a POST operation.
	@returns: The raw data returned from teh request, as a string.
	"""
	headers = {"Content-Type":"application/json"}
	headers["X-CDD-Token"] = token
	query = urllib2.Request(url,headers=headers)
	resp = ''
	if post == True:
		#There is an issue with urllib2 and https which makes this line necessary, otherwise urllib2 uses a GET even though it should be recognising that a POST is required
		query.get_method = lambda: 'POST'
	try:
		resp = urllib2.urlopen(query).read()
	except urllib2.HTTPError, e:
		print e.fp.read()
	except Exception, e:
		print e	
	return resp

def runSearch(appui,searchId,vaultId,projectIDs,token):
	"""Perform a search of the CDD Vault.
	@param: appui: The StarDrop application interface.
	@param: searchId: The unique ID of the search to perform.
	@param: vaultId: The unique ID of the vault to be searched.
	@param: token: The API token to be used, as a string.
	@returns: A python reprseentation of a StarDrop data set.
	"""
	if appui:
		progress = PythonQt.SimpleProgress()
		progress.setWindowTitle("Running search")
		progress.show()
		timer = 60
		progress.setProgress(timer)
	#Run the search
	if len(projectIDs) > 0:
		pids = ','.join(map(str, projectIDs))
		urlExport = cddconfig.url + '/%i/exports?search=%i&projects=%s' % (vaultId,searchId,pids)
		print urlExport
	else:
		urlExport = cddconfig.url + '/%i/exports?search=%i' % (vaultId,searchId)
	resp = callServer(urlExport,token,True)
	result = json.loads(resp)
	exportId = result.get("id",0)
	ready = False
	if exportId == 0:
		if appui:
			appui.showError("Unable to run search the search")
			progress.hide()
		return
	if appui:
		timer == 70
	while ready == False:
		#Poll server until search is ready to retrieve
		if appui:
			progress.setProgress(timer % 100)
		urlCheck = cddconfig.url + '/%i/export_progress/%i' % (vaultId,exportId)
		resp = callServer(urlCheck,token)
		result = json.loads(resp)
		if result.get("status","new") == "finished":
			ready = True
		else:
			time.sleep(1)
			if appui:
				timer += 5
	#Download search results and create dataset	
	urlDownload = cddconfig.url + '/%i/exports/%i' % (vaultId,exportId)
	resp = callServer(urlDownload,token)
	dataset =None
	print resp
	if resp != '':
		dataset = CSVHelper.createDataSet(resp,cddconfig)
		if dataset.rowSz() == 0:
			if appui:
				progress.hide()
			return
	if appui:
		progress.hide()
	return dataset

def getVaultID(appui, progress, progressvalue):	
	"""Find out the vault to access.
	@param: appui: The StarDrop application interface.
	@param: progressvalue: The starting value when a progress bar is shown.
	@param: progressdialogue: A dialogue we'll be using to show progress.
	"""
	progress.setWindowTitle("Finding available vaults")
	progress.show()
	progress.setProgress(progressvalue)
	vaults = {}

	try:
		resp = callServer(cddconfig.url,token)
		results = json.loads(resp)
		progress.setProgress(progressvalue + 10)
		#Find our Vault ID
		for vault in results:
			vaults[vault["name"]] = vault["id"]	
	except:
		pass
		
	if len(vaults) == 0:
		progress.hide()
		return invalidvaultid
	elif len(vaults) == 1:
		print 'Using vault ' + vaults.keys()[0]
		return (vaults.values()[0], vaults.keys()[0])
	else:
		progress.hide()
		dlg = PythonQt.SimpleList()
		dlg.setAvailableItems(vaults.keys())
		dlg.setWindowTitle("CDD Vault Access")
		dlg.setLabel("Select a vault to search:")
		if not dlg.run():
			return usercancelled
		return (vaults[dlg.getSelectedItem()], dlg.getSelectedItem())

def getProjectIDs(appui, vaultId, progress, progressvalue):
	"""Find out the project(s) to search from this vault.
	@param: appui: The StarDrop application interface.
	@param: vaultId: The vault in which to locate projects.
	@param: progressvalue: The starting value when a progress bar is shown.
	@param: progressdialogue: A dialogue we'll be using to show progress.
	"""
	urlProject = cddconfig.url + '/%i/projects' % (vaultId[0])
	progress.setWindowTitle("Finding projects in: " + vaultId[1])
	progress.show()
	progress.setProgress(progressvalue)
	projects = {}
	projectIDs = []
	try:
		resp = callServer(urlProject,token)
		results = json.loads(resp)
		progress.setProgress(progressvalue + 10)
		for project in results:
			projects[project["name"]] = project["id"]	
	except:
		pass
		
	if len(projects) == 0:
		progress.hide()
		return invalidprojectids
	elif len(projects) == 1:
		print 'Searching project ' + projects.keys()[0]
		return projectIDs.append(projects.values()[0])
	else:
		progress.hide()
		dlg = PythonQt.SimpleList()
		dlg.setAvailableItems(projects.keys())
		dlg.setMultipleSelections()
		dlg.setWindowTitle("CDD Vault Access")
		dlg.setLabel("Select project(s) to search:")
		if not dlg.run():
			return usercancelledproject
		
		for item in dlg.getSelectedItems():
			projectIDs.append(projects[item])
		return projectIDs
	
def getSearchID(appui, vaultId, progress, progressvalue):		
	"""Find out the search to use form those available in this vault.
	@param: appui: The StarDrop application interface.
	@param: vaultId: The vault in which to locate projects.
	@param: progressvalue: The starting value when a progress bar is shown.
	@param: progressdialogue: A dialogue we'll be using to show progress.
	"""

	urlSearch = cddconfig.url + '/%i/searches' % (vaultId[0])
	progress.setWindowTitle("Finding saved searches in: " + vaultId[1])
	progress.show()
	progress.setProgress(progressvalue)
	searches = {}
	try:
		resp = callServer(urlSearch,token)
		results = json.loads(resp)
		progress.setProgress(progressvalue + 10)
		for search in results:
			searches[search["name"]] = search["id"]
	except:
		pass
		
	if len(searches) == 0:
		progress.hide()
		return invalidsearchid
	#Ask the user to select/confirm a search to run	
	dlg = PythonQt.SimpleList()
	dlg.setAvailableItems(searches.keys())
	dlg.setWindowTitle("CDD Vault Access")
	dlg.setLabel("Select a saved search to run:")
	if not dlg.run():
		return usercancelled
		
	return (searches.get(dlg.getSelectedItem(),0), dlg.getSelectedItem())

def connectToVault(appui):
	"""Respond to a user request to connect to the CDD Vault.
	@param: appui: The StarDrop application interface.
	"""
	global token
	print 'Connecting to vault'
	if token == '':
		changeToken(appui)
		if token == '':
			return
	
	progress = PythonQt.SimpleProgress()
	vaultId = getVaultID(appui, progress, 10)
	if vaultId == invalidvaultid:
		appui.showError("No vaults found")
		return
	elif vaultId == usercancelled:
		return
		
	searchId = getSearchID(appui, vaultId, progress, 30)
	if searchId == invalidsearchid:
		appui.showError("No searches found")
		return
	elif searchId == usercancelled:
		return
		
	projectIDs = getProjectIDs(appui, vaultId, progress, 50 )
	if projectIDs == invalidprojectids:
		appui.showError("No projects available to search")
		return
	elif projectIDs == usercancelled:
		return

	dataset = runSearch(appui,searchId[0],vaultId[0],projectIDs,token)
	if dataset:
		appui.exportDS(searchId[1],dataset.pack())
		#TODO need to store the project IDs as part of the Refresh information, eg "projects": projectIDs)
		appui.setRefreshKey("","CDD",{"search":`searchId[0]`, "vault":`vaultId[0]`, "projects":projectIDs, "token":token})
	else:
		appui.showError("Unable to run the search against the chosen project(s)")
			
def changeToken(appui):
	"""Respond to a user request to change the API token being used.
	@param: appui: The StarDrop application interface.
	"""
	global token
	newToken = appui.query("CDD Token","Please enter your CDD token:", "")
	if newToken == '':
		return
	token = newToken
	cddconfig.setToken(newToken)
	
	
	
	
	
