"""
StarDropScriptFinder.py - class for finding python scripts for StarDrop

Use of this code is subject to the terms of the StarDrop License Agreement and
use, copying or redistribution of this file for use by other persons or
organisations is prohibited

Copyright(C) Optibrium Ltd 2016
stardrop-support@optibrium.com
"""

import sys
import os
import inspect
import imp
from HelperFunctions import extract_filename


class StarDropScriptFinder:
    def __init__(self):
        self.directory = ""
        self.function_name = ""
        self.launch_function_name = ""
        self.ignore_files = []

    def __get_scripts(self):
        """
        Return full path to python scripts in directory, including subdirectories.

        :rtype: List containing python script file paths
        """
        scripts = []
        for root, sub_dirs, file_names in os.walk(self.directory):
            for file_name in file_names:
                path = os.path.join(root, file_name)
                if path not in self.ignore_files and file_name.endswith('.py'):
                    scripts.append(os.path.join(root, file_name))

        return scripts

    def __analyse(self, path):
        """
        Loads the script in path as a module and tests for the function attribute
        function_name.

        :param path: Path to a python script
        :return:     Boolean indicating whether the script contains function_name
        """
        file_name = extract_filename(path)
        file_dir = os.path.dirname(path)
        sys.path.append(file_dir)

        result = False
        module = imp.load_source(file_name, path)

        if hasattr(module, self.function_name):
            function = getattr(module, self.function_name)
            if inspect.isfunction(function):
                definitions = function()
                if type(definitions) is list:
                    for definition in definitions:
                        if 'script_name' in definition and 'callback' in definition:
                            result = True
                        else:
                            attribute = 'script_name' if 'callback' in definition else 'callback'
                            print "error: StarDrop definition in script " + path + " does not contain a %s" % attribute
                else:
                    print "error: StarDrop definition in script " + path + " is not a list"
        elif hasattr(module, self.launch_function_name):
            result = True

        sys.path.pop()
        del module
        return result

    def search(self):
        """
        Searches a directory, including subdirectories, for python scripts that have
        function_name attribute defined.

        :return: List of scripts with function_name attribute defined
        """
        scripts = []
        for s in self.__get_scripts():
            try:
                if self.__analyse(s):
                    scripts.append(s)
            except Exception as e:
                print "error in script " + s + ": " + str(e)

        return scripts
