"""
layout.py - User layout engine configuration script

The following line is important for maintaining StarDrop's script interface. DO NOT REMOVE
UpgradeVersion63

Use of this code is subject to the terms of the StarDrop License Agreement and
use, copying or redistribution of this file for use by other persons or
organisations is prohibited

Copyright(C) Optibrium Ltd 2016
stardrop-support@optibrium.com
"""

import os
import sys
import imp
import scriptenv

home = appui.homeDir()  # user's ~ dir
os.chdir(home)

py_dir = os.path.join(os.getcwd(), 'py')
sys.path.append(os.getcwd())
sys.path.append(os.path.join(py_dir, 'Utils'))

from StarDropScriptFinder import StarDropScriptFinder
from HelperFunctions import extract_filename

try:
    layoutengine = sys.modules["__main__"].layoutengine
    print 'loaded layoutengine'
except Exception as e:
    print str(e)

STARDROP_FUNCTION = 'get_stardrop_layout_definitions'
LAYOUT_LOCATION_FILE = os.path.join(py_dir, 'LayoutLocations.txt')

# Find and collect scripts with 'STARDROP_FUNCTION' defined
modules = []
layout_dirs = [py_dir]

# Collect other directories to search for
try:
    with open(LAYOUT_LOCATION_FILE, 'r') as f:
        directories = f.readlines()
        for directory in directories:
            if directory not in layout_dirs:
                layout_dirs.append(directory)
except IOError as e:
    print 'no other layout directories defined'
except Exception as e:
    print 'error opening layout directories file: ' + LAYOUT_LOCATION_FILE

finder = StarDropScriptFinder()
finder.function_name = STARDROP_FUNCTION

# Ignore top-level scripts user.py and layout.py
user_file = os.path.join(py_dir, 'user.py')
layout_file = os.path.join(py_dir, 'layout.py')
env_file = os.path.join(py_dir, 'scriptenv.py')

finder.ignore_files = [user_file, layout_file, env_file]

for layout_dir in layout_dirs:
    finder.directory = layout_dir
    print 'searching for StarDrop layouts in ' + layout_dir

    layouts = finder.search()
    if layouts:
        for layout in layouts:
            layout_name = extract_filename(layout)
            layout_dir = os.path.dirname(layout)
            sys.path.append(layout_dir)

            try:
                module = imp.load_source(layout_name, layout)
                function = getattr(module, STARDROP_FUNCTION)
                definitions = function()
                for definition in definitions:
                    modules.append(definition)
                print "loaded " + layout_name
            except:
                print "failed to load " + layout_name
                sys.path.pop()
    else:
        print 'no StarDrop layouts found'

# Add scripts to be accessed from UI menu and build a map of callbacks
for module in modules:
    try:
        name = module['script_name']
        callback = module['callback']
        scriptenv.add_layout_script(layoutengine, name, callback)
    except:
        print 'failed to add name/callback for a module'


def layoutScript(index):
    if index == 0:
        print 'called layout script 0'
    elif scriptenv.has_layout_script(index):
        scriptenv.run_layout_script(index, appui, layoutengine)
    else:
        print 'layout ' + repr(index)


layoutengine.connect("scriptLayout(int)", layoutScript)
