"""
scriptenv.py - maintains the current scripting environment

The following line is important for maintaining StarDrop's script interface. DO NOT REMOVE
UpgradeVersion63

Use of this code is subject to the terms of the StarDrop License Agreement and
use, copying or redistribution of this file for use by other persons or
organisations is prohibited

Copyright(C) Optibrium Ltd 2016
stardrop-support@optibrium.com
"""

__script_map = {}
__script_index = 1

__layout_map = {}
__layout_index = 1


def add_script(appui, name, callback):
    """
    Adds a script and callback to the menu

    :param appui:       StarDrop appui object
    :param name:        Name of script
    :param callback:    Callback function to invoke when script selected
    :return:
    """
    global __script_index, __script_map
    __script_map[__script_index] = {
        'script_name': name,
        'callback': callback
    }

    appui.addScript(__script_index, name)
    __script_index += 1


def remove_script(appui, name):
    """
    Removes a script from the menu

    :param appui:   StarDrop appui object
    :param name:    Name of script to remove
    """
    global __script_index, __script_map

    for index, script in __script_map.iteritems():
        if script['script_name'] == name:
            del __script_map[index]
            appui.removeMenuItems(name)
            break


def has_script(index):
    """
    Tests whether the index of the selected action has
    an associated callback

    :param index:   Index of selected action
    :return:        Whether the action has a callback
    """
    global __script_map
    return index in __script_map


def run_script(index, appui):
    """
    Runs the script added with the index

    :param index:   Index of the script to run
    :param appui:   StarDrop appui object
    """
    global __script_map

    if index in __script_map:
        __script_map[index]['callback'](appui)


def add_layout_script(layoutengine, name, callback):
    """
    Adds a layout script and callback to the menu

    :param layoutengine:       StarDrop appui object
    :param name:        Name of layout script
    :param callback:    Callback function to invoke when script selected
    :return:
    """
    global __layout_index, __layout_map
    __layout_map[__layout_index] = {
        'script_name': name,
        'callback': callback
    }

    layoutengine.addLayoutScript(__layout_index, name)
    __layout_index += 1


# We currently cannot provide this
#def remove_layout_script(appui, name):
#    """
#    Removes a layout script from the menu
#
#    :param appui:   StarDrop appui object
#    :param name:    Name of layout script to remove
#    """
#    global __layout_index, __layout_map
#    return


def has_layout_script(index):
    """
    Tests whether the index of the selected layout action has
    an associated callback

    :param index:   Index of selected layout action
    :return:        Whether the action has a callback
    """
    global __layout_map
    return index in __layout_map


def run_layout_script(index, appui, layoutengine):
    """
    Runs the layout script added with the index

    :param index:   Index of the layout script to run
    :param appui:   StarDrop appui object
    """
    global __layout_map

    if index in __layout_map:
        __layout_map[index]['callback'](appui, layoutengine)
