"""
user.py - user configuration script for StarDrop

The following line is important for maintaining StarDrop's script interface. DO NOT REMOVE
UpgradeVersion63

Use of this code is subject to the terms of the StarDrop License Agreement and
use, copying or redistribution of this file for use by other persons or
organisations is prohibited

Copyright(C) Optibrium Ltd 2016
stardrop-support@optibrium.com
"""

import os
import sys
import imp
import scriptenv

home = appui.homeDir()  # user's ~ dir
os.chdir(home)

py_dir = os.path.join(os.getcwd(), 'py')
py_install_dir = os.path.join(appui.installDir(), 'py')
sys.path.append(os.getcwd())
sys.path.append(os.path.join(py_dir, 'Utils'))

from StarDropScriptFinder import StarDropScriptFinder
from HelperFunctions import extract_filename

STARDROP_FUNCTION = 'get_stardrop_definitions'
MENU_NAME_DEFINTION = 'CustomMenuName'
LAUNCH_SCRIPT_DEFINITION = 'LaunchScriptFunction'
SCRIPT_LOCATION_FILES = [os.path.join(py_dir, 'ScriptLocations.txt'),os.path.join(py_install_dir,'ScriptLocations.txt')]

# Find and collect scripts with 'STARDROP_FUNCTION' defined
modules = []
script_dirs = [py_dir]
menu_names = ['Custom Scripts']
launch_function = None

# Collect other directories to search for
for script_location_file in SCRIPT_LOCATION_FILES:
    try:
        with open(script_location_file, 'r') as f:
            directories = f.readlines()
            for directory in directories:
                if directory not in script_dirs:
                    script_dirs.append(directory)
    except IOError as e:
        print 'no other script directories defined in: ' + script_location_file
    except Exception as e:
        print 'error opening script directories file: ' + script_location_file

finder = StarDropScriptFinder()
finder.function_name = STARDROP_FUNCTION
finder.launch_function_name = LAUNCH_SCRIPT_DEFINITION

# Ignore top-level scripts user.py and layout.py
user_file = os.path.join(py_dir, 'user.py')
layout_file = os.path.join(py_dir, 'layout.py')
env_file = os.path.join(py_dir, 'scriptenv.py')

finder.ignore_files = [user_file, layout_file, env_file]

for script_dir in script_dirs:
    finder.directory = script_dir
    print 'searching for StarDrop scripts in ' + script_dir

    scripts = finder.search()
    if scripts:
        for script in scripts:
            script_name = extract_filename(script)
            script_dir = os.path.dirname(script)
            sys.path.append(script_dir)

            try:
                module = imp.load_source(script_name, script)
                try:
                    function = getattr(module, STARDROP_FUNCTION)
                    definitions = function()
                    for definition in definitions:
                        modules.append(definition)
                except:
                    print "no scripts defined in " + script_name

                if hasattr(module, MENU_NAME_DEFINTION):
                    name = getattr(module, MENU_NAME_DEFINTION)
                    menu_names.append(name)
                if hasattr(module, LAUNCH_SCRIPT_DEFINITION):
                    launch_function = getattr(module, LAUNCH_SCRIPT_DEFINITION)

                print "loaded " + script_name
            except Exception as e:
                print "failed to load " + script_name
                sys.path.pop()
    else:
        print 'no StarDrop scripts found'

# Add scripts to be accessed from UI menu and build a map of callbacks
for module in modules:
    try:
        name = module['script_name']
        callback = module['callback']
        scriptenv.add_script(appui, name, callback)
    except Exception as e:
        print 'failed to add name/callback for a module'

# Set the menu title
try:
    appui.setScriptMenuTitle(menu_names[-1])
except:
    pass
    
# Specify the function to call on launch
def launchFunction(params):
    if launch_function:
        launch_function(appui,params)
    else:
        print 'No launch script specified'

appui.connect("scriptWithParams(const QStringList&)", launchFunction)



def script(index):
    if index == 0:
        print 'called script 0'
    elif scriptenv.has_script(index):
        scriptenv.run_script(index, appui)
    else:
        print 'script ' + repr(index)


appui.connect("script(int)", script)

